﻿namespace UXGridViewTest
{
    #region Usings

    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.ComponentModel;
    using System.Diagnostics;
    using System.Windows.Threading;

    #endregion

    public class MainPageViewModel : INotifyPropertyChanged
    {
        public MainPageViewModel()
        {
            _timer.Tick += timer_Tick;
            _timer.Interval = TimeSpan.FromSeconds(1);            
        }

        private readonly DispatcherTimer _timer = new DispatcherTimer();

        public void GetData()
        {
            IsBusy = true;

            _timer.Start();
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            _timer.Stop();
            var pasSummariesList = new List<PasSummary>();

            pasSummariesList.Add(
                new PasSummary { Description = "Description 1", Function = "Function 1", Number = 001 });
            PasSummaries = new ObservableCollection<PasSummary>(pasSummariesList);
            
            IsBusy = false;
        }

        #region IsBusy Property

        /// <summary>
        /// Description
        /// </summary>
        private bool _isBusy;

        /// <summary>
        /// Description
        /// </summary>
        public bool IsBusy
        {
            get
            {
                return _isBusy;
            }
            set
            {
                //ignore if values are equal
                if (value == _isBusy)
                {
                    return;
                }

                _isBusy = value;
                InvokePropertyChanged("IsBusy");
            }
        }

        #endregion IsBusy Property

        #region SelectedDate Property

        /// <summary>
        /// Description
        /// </summary>
        private DateTime? _selectedDate = DateTime.Now;

        /// <summary>
        /// Description
        /// </summary>
        public DateTime? SelectedDate
        {
            get
            {
                return _selectedDate;
            }
            set
            {
                //ignore if values are equal
                if (value == _selectedDate)
                {
                    return;
                }

                _selectedDate = value;
                InvokePropertyChanged("SelectedDate");

                GetData();
            }
        }

        #endregion SelectedDate Property        

        #region PasSummaries Property

        /// <summary>
        /// PasSummaries
        /// </summary>
        private ObservableCollection<PasSummary> _pasSummaries;

        /// <summary>
        /// PasSummaries
        /// </summary>
        public ObservableCollection<PasSummary> PasSummaries
        {
            get
            {
                return _pasSummaries;
            }
            set
            {
                //ignore if values are equal
                if (value == _pasSummaries)
                {
                    return;
                }

                _pasSummaries = value;
                InvokePropertyChanged("PasSummaries");
            }
        }

        #endregion PasSummaries Property

        #region INotifyPropertyChanged event

        ///<summary>
        ///Occurs when a property value changes.
        ///</summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Raises the <see cref="PropertyChanged"/> event for
        /// a given property.
        /// </summary>
        /// <param name="propertyName">The name of the changed property.</param>
        protected void InvokePropertyChanged(string propertyName)
        {
            //validate the property name in debug builds
            VerifyProperty(propertyName);

            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        /// <summary>
        /// Verifies whether the current class provides a property with a given
        /// name. This method is only invoked in debug builds, and results in
        /// a runtime exception if the <see cref="InvokePropertyChanged"/> method
        /// is being invoked with an invalid property name. This may happen if
        /// a property's name was changed but not the parameter of the property's
        /// invocation of <see cref="InvokePropertyChanged"/>.
        /// </summary>
        /// <param name="propertyName">The name of the changed property.</param>
        [Conditional("DEBUG")]
        private void VerifyProperty(string propertyName)
        {
            var type = GetType();

            //look for a *public* property with the specified name
            var pi = type.GetProperty(propertyName);

            //there is no matching property - notify the developer
            var msg = "InvokePropertyChanged was invoked with invalid property name {0}: ";
            msg += "{0} is not a public property of {1}.";
            msg = String.Format(msg, propertyName, type.FullName);

            Debug.Assert(pi != null, msg);
        }

        #endregion
    }
}