﻿namespace UXGridViewTest
{
    #region Usings

    using System;
    using System.ComponentModel;
    using System.Diagnostics;

    #endregion

    public class PasSummary : INotifyPropertyChanged
    {
        #region Number Property

        /// <summary>
        /// Description
        /// </summary>
        private int _number;


        /// <summary>
        /// Description
        /// </summary>
        public int Number
        {
            get
            {
                return _number;
            }
            set
            {
                //ignore if values are equal
                if (value == _number) return;

                _number = value;
                InvokePropertyChanged("Number");
            }
        }

        #endregion Number Property

        #region Description Property

        /// <summary>
        /// Description
        /// </summary>
        private string _description;


        /// <summary>
        /// Description
        /// </summary>
        public string Description
        {
            get
            {
                return _description;
            }
            set
            {
                //ignore if values are equal
                if (value == _description) return;

                _description = value;
                InvokePropertyChanged("Description");
            }
        }

        #endregion Description Property

        #region Function Property

        /// <summary>
        /// Function
        /// </summary>
        private string _function;


        /// <summary>
        /// Function
        /// </summary>
        public string Function
        {
            get
            {
                return _function;
            }
            set
            {
                //ignore if values are equal
                if (value == _function) return;

                _function = value;
                InvokePropertyChanged("Function");
            }
        }

        #endregion Function Property

        #region TotalHours Property

        /// <summary>
        /// Description
        /// </summary>
        private Decimal _totalHours;


        /// <summary>
        /// Description
        /// </summary>
        public Decimal TotalHours
        {
            get
            {
                return _totalHours;
            }
            set
            {
                //ignore if values are equal
                if (value == _totalHours) return;

                _totalHours = value;
                InvokePropertyChanged("TotalHours");
            }
        }

        #endregion TotalHours Property      

        #region INotifyPropertyChanged event

        ///<summary>
        ///Occurs when a property value changes.
        ///</summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Raises the <see cref="PropertyChanged"/> event for
        /// a given property.
        /// </summary>
        /// <param name="propertyName">The name of the changed property.</param>
        protected void InvokePropertyChanged(string propertyName)
        {
            //validate the property name in debug builds
            VerifyProperty(propertyName);

            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        /// <summary>
        /// Verifies whether the current class provides a property with a given
        /// name. This method is only invoked in debug builds, and results in
        /// a runtime exception if the <see cref="InvokePropertyChanged"/> method
        /// is being invoked with an invalid property name. This may happen if
        /// a property's name was changed but not the parameter of the property's
        /// invocation of <see cref="InvokePropertyChanged"/>.
        /// </summary>
        /// <param name="propertyName">The name of the changed property.</param>
        [Conditional("DEBUG")]
        private void VerifyProperty(string propertyName)
        {
            var type = GetType();

            //look for a *public* property with the specified name
            var pi = type.GetProperty(propertyName);

            //there is no matching property - notify the developer
            var msg = "InvokePropertyChanged was invoked with invalid property name {0}: ";
            msg += "{0} is not a public property of {1}.";
            msg = String.Format(msg, propertyName, type.FullName);

            Debug.Assert(pi != null, msg);
        }

        #endregion
    }
}